
/**
 * @vue-cdk/tooltip
 * Version: 0.10.0,
 * (c) Christian Kienle, 2020
 * LICENCE: MIT
 * https://github.com/ChristianKienle/cdk
*/
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('@vue-cdk/pluginify'), require('@vue-cdk/popover/src/popover'), require('vue'), require('@vue-cdk/popover/src/content.vue')) :
  typeof define === 'function' && define.amd ? define(['@vue-cdk/pluginify', '@vue-cdk/popover/src/popover', 'vue', '@vue-cdk/popover/src/content.vue'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, global['@vue-cdk/tooltip'] = factory(global.pluginify, global.Popover, global.Vue, global.CPopoverContent));
}(this, (function (pluginify, Popover, Vue, CPopoverContent) { 'use strict';

  function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

  var pluginify__default = /*#__PURE__*/_interopDefaultLegacy(pluginify);
  var Popover__default = /*#__PURE__*/_interopDefaultLegacy(Popover);
  var Vue__default = /*#__PURE__*/_interopDefaultLegacy(Vue);
  var CPopoverContent__default = /*#__PURE__*/_interopDefaultLegacy(CPopoverContent);

  // @ts-check

  /**
   * @template {keyof DocumentEventMap} K
   * @param {K} type
   * @param {HTMLElement | Document} subject
   * @param {(event: DocumentEventMap[K]) => void} handler
   */
  var createEventListener = function createEventListener(type, subject, handler) {
    var _isOn = false;
    return {
      isOn: function isOn() {
        return _isOn;
      },
      on: function on() {
        if (_isOn) {
          throw Error('cannot enable a listener that is already enabled');
        }

        _isOn = true;
        subject.addEventListener(type, handler);
        return this;
      },
      off: function off() {
        if (_isOn === false) {
          throw Error('cannot disable a listener that is not enabled');
        }

        _isOn = false;
        subject.removeEventListener(type, handler);
        return this;
      }
    };
  };

  // @ts-check
  /**
   * @param {() => {}} callback
   * @param {{element?: HTMLElement }} options
   */

  var createOnEscListener = (function (callback) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var element = options.element || document;
    return createEventListener('keyup', element, function (event) {
      if (event.key === 'Escape') {
        callback();
      }
    });
  });

  function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

  function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

  function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
  var script = {
    name: 'TooltipContent',
    functional: true,
    render: function render(h, context) {
      var scopedSlots = {
        default: context.scopedSlots.default
      };

      var attrs = _objectSpread(_objectSpread({}, context.data.attrs), {}, {
        role: 'tooltip'
      });

      var data = _objectSpread(_objectSpread({}, context.data), {}, {
        scopedSlots: scopedSlots,
        attrs: attrs
      });

      return h(CPopoverContent__default['default'], data, [context.children]);
    }
  };

  var render = function render() {};

  script.render = render;

  var script$1 = {
    name: 'Tooltip',
    components: {
      Popover: Popover__default['default']
    },
    props: {
      noArrow: {
        type: Boolean,
        default: false
      },
      visible: {
        type: Boolean,
        default: null
      },
      theme: {
        type: String,
        default: null
      },
      text: {
        type: String,
        default: null
      },
      placement: {
        type: String,
        default: 'top'
      }
    },
    computed: {
      popoverTheme: function popoverTheme() {
        var theme = this.theme;

        if (theme == null) {
          return null;
        }

        return "tooltip-".concat(this.theme);
      }
    },
    watch: {
      visible: function visible(_visible) {
        var $_popover = this.$_popover;

        if ($_popover == null) {
          return;
        }

        if (_visible) {
          $_popover.show();
        } else {
          $_popover.hide();
        }
      },
      popoverTheme: function popoverTheme(_popoverTheme) {
        var $_popover = this.$_popover;

        if ($_popover == null) {
          return;
        }

        $_popover.theme = _popoverTheme;
      }
    },
    beforeUpdate: function beforeUpdate() {
      if (this.$el == null) {
        return;
      }

      this.turnListenersOff();
    },
    updated: function updated() {
      var $el = this.$el;

      if ($el == null) {
        return;
      }

      this.setupListeners();
      this.turnListenersOn();
    },
    destroy: function destroy() {
      this.$_popover.$destory();
      this.turnListenersOff();
    },
    mounted: function mounted() {
      var _this = this;

      var target = function target() {
        return _this.$el;
      };

      var theme = this.popoverTheme;
      this.$_escListener = createOnEscListener(function () {
        return _this.handleESC();
      });
      var tooltipVM = this;
      this.$_popover = new Vue__default['default']({
        components: {
          Popover: Popover__default['default']
        },
        data: function data() {
          return {
            theme: theme,
            noArrow: tooltipVM.noArrow,
            visible: tooltipVM.visible,
            placement: tooltipVM.placement
          };
        },
        methods: {
          show: function show() {
            this.$refs.popover.show();
            this.visible = true;
          },
          hide: function hide() {
            this.$refs.popover.hide();
            this.visible = false;
          }
        },
        render: function render(h) {
          var _this2 = this;

          var props = {
            target: target,
            withArrow: !this.noArrow,
            theme: this.theme,
            placement: this.placement
          };
          var on = {
            'update:visible': function updateVisible(newVisible) {
              if (_this2.visible != null) {
                return;
              }

              _this2.visible = newVisible;
            }
          };
          var scopedSlots = {
            default: function _default(popoverProps) {
              var text = tooltipVM.text,
                  $scopedSlots = tooltipVM.$scopedSlots;

              if (text != null) {
                return h(script, {}, [text]);
              }

              return $scopedSlots.content(popoverProps);
            }
          };
          return h(Popover__default['default'], {
            ref: 'popover',
            on: on,
            parent: tooltipVM,
            props: props,
            scopedSlots: scopedSlots
          });
        }
      }).$mount();
      this.$forceUpdate();
    },
    methods: {
      setupListeners: function setupListeners() {
        var _this3 = this;

        var visible = this.visible;

        this.$_mouseEnterListener = createEventListener('mouseenter', this.$el, function () {
          _this3.handleMouseEnter();
        });
        this.$_mouseLeaveListener = createEventListener('mouseleave', this.$el, function () {
          _this3.handleMouseLeave();
        });
        this.$_blurListener = createEventListener('blur', this.$el, function () {
          return _this3.handleBlur();
        });
        this.$_focusListener = createEventListener('focus', this.$el, function () {
          return _this3.handleFocus();
        });
      },
      turnListenersOff: function turnListenersOff() {
        this.$_mouseEnterListener != null && this.$_mouseEnterListener.isOn() && this.$_mouseEnterListener.off();
        this.$_mouseLeaveListener != null && this.$_mouseLeaveListener.isOn() && this.$_mouseLeaveListener.off();
        this.$_blurListener != null && this.$_blurListener.isOn() && this.$_blurListener.off();
        this.$_focusListener != null && this.$_focusListener.isOn() && this.$_focusListener.off();
      },
      turnListenersOn: function turnListenersOn() {
        var visible = this.visible;

        this.$_mouseEnterListener.isOn() === false && this.$_mouseEnterListener.on();
        this.$_mouseLeaveListener.isOn() === false && this.$_mouseLeaveListener.on();
        this.$_blurListener.isOn() === false && this.$_blurListener.on();
        this.$_focusListener.isOn() === false && this.$_focusListener.on();
      },
      handleMouseEnter: function handleMouseEnter() {
        this.$_popover.show();
        this.$_escListener.isOn() === false && this.$_escListener.on();
      },
      handleMouseLeave: function handleMouseLeave() {
        this.$_popover.hide();
        this.$_escListener.isOn() && this.$_escListener.off();
      },
      handleBlur: function handleBlur() {
        this.$_popover.hide();
        this.$_escListener.isOn() && this.$_escListener.off();
      },
      handleFocus: function handleFocus() {
        this.$_popover.show();
        this.$_escListener.isOn() === false && this.$_escListener.on();
      },
      handleESC: function handleESC() {
        this.$_popover.hide();
        this.$_escListener.isOn() && this.$_escListener.off();
      }
    },
    render: function render() {
      return this.$slots.default;
    }
  };

  var render$1 = function render() {};

  script$1.render = render$1;

  var index = pluginify__default['default'](script$1, script);

  return index;

})));
